@@ -9,7 +9,7 @@ Created on Tue Mar 23 15:44:39 2021
 
 from data_science_helper import helper_general as hg
 
-
+import statistics
 import json
 import os
 import math
@@ -34,11 +34,15 @@ from data_science_helper.model import general as g
 from data_science_helper.model import neg_bagging_fraction__lgb_model as nbf_lgb_model
 from data_science_helper.model import scale_pos_weight__lgb_model as spw_lgb_model
 from data_science_helper.model import custom_bagging__lgb_model as cb_lgb_model
-
+from sklearn.model_selection import StratifiedKFold   
 from sklearn.metrics import f1_score
 
 from data_science_helper.model import lgb_model as l
 
+from sklearn.metrics import precision_recall_fscore_support
+from sklearn.metrics import accuracy_score, auc, average_precision_score, confusion_matrix, roc_curve, precision_recall_curve
+from numpy import interp
+from sklearn.metrics import  roc_auc_score, average_precision_score 
 #import src.Prj_Core.core_helper.model.neg_bagging_fraction__lgb_model as nbf_lgb_model
 #import src.Prj_Core.core_helper.model.scale_pos_weight__lgb_model as spw_lgb_model
 #import src.Prj_Core.core_helper.model.custom_bagging__lgb_model as cb_lgb_model
@@ -47,7 +51,7 @@ from data_science_helper.model import lgb_model as l
 #import model.scale_pos_weight__lgb_model as spw_lgb_model
 #import model.custom_bagging__lgb_model as cb_lgb_model
 
-def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=None,y_test=None,params=None,max_iter_cls=None, cls_ini_opt=[] ,enable_cls_opt = False, enable_umbral_opt=False,metric='average_precision',num_rounds=10000,early_stop=400,log=0,api="train_api",sesgo=0.15,random_state=42,url=None,print_consola=True):
+def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=None,y_test=None,params=None,max_iter_cls=None, cls_ini_opt=[] ,enable_cls_opt = False, enable_umbral_opt=False,metric='average_precision',num_rounds=10000,early_stop=400,log=0,api="train_api",sesgo=0.15,random_state=42,url=None, cv=False,cv_n_split=None, print_consola=True):
     start = time.time()
     
     if url is not None:
@@ -134,14 +138,35 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
         
         print("----------- FIN enable_cls_opt = True ---------------------")
         
-
-   
+        
     model ,  y_prob_uno_train, y_prob_uno_test    = l.modelar(**args)
     
-    kpis_train = generar_reporte(model,y_prob_uno_train,X_train,y_train,None,None,print_consola)
+    if early_stop is None:
+        num_rounds_cv = num_rounds 
+    else:
+        num_rounds_cv = model.best_iteration()
+        
+    
+    if cv : 
+        
+        if url is None:
+            print ("SE NECESITA UNA URL PARA PROCESAR CROSS VALIDATION")
+            
+        else:
+        
+            args_cv = { 'X_train':X_train, 'y_train': y_train, 'X_test': X_test, 'y_test': y_test, 'params':params,
+                        "metric":metric,"num_rounds":num_rounds_cv,"random_state":random_state,
+                        "api":api , "url":url , "cv_n_split" :cv_n_split
+                        }      
+            
+            generate_cv_result(**args_cv)    
 
+    
+    kpis_train = generar_reporte(model,y_prob_uno_train,X_train,y_train,None,None,print_consola)
+    
     kpis_test = generar_reporte(model,y_prob_uno_test,X_test,y_test,None,url,print_consola)
     
+        
 
     umbral_opt=0
     kpis_opt = (0,0,0,0,0)
@@ -155,6 +180,213 @@ def modelar_clasificacion_binaria(strategy="", X_train=None,y_train=None,X_test=
     return model ,  kpis_train, kpis_test , kpis_opt , umbral_opt , list_cls_ini
 
 
+
+def generate_cv_result(X_train=None,y_train=None,X_test=None,y_test=None,api=None, num_rounds = None , metric=None, 
+                       params =None ,  url=None, cv_n_split=None, random_state=42):
+    
+   print(X_train.shape)
+   X_t_ = X_train.append(X_test)
+   y_ = y_train.append(y_test)
+   
+   if cv_n_split is None:
+       test_size = get_test_size(X_t_)
+       n_s = round(1/test_size)
+   else:
+       n_s = cv_n_split
+
+   folds = StratifiedKFold(n_splits=n_s, shuffle=True, random_state=random_state) 
+   print("##########################################")
+   print("params: : ", params)
+   print("num_rounds_cv : ",num_rounds)
+   print("##########################################")
+
+   path_cv = os.path.join(url,"cv")  
+   hg.validar_directorio(path_cv)  
+   
+   i = 0   
+   for train_idx, valid_idx in folds.split(X_t_, y_):
+       
+       X_train_cv, y_train_cv  = X_t_.iloc[train_idx], y_.iloc[train_idx]
+       X_valid_cv , y_valid_cv = X_t_.iloc[valid_idx], y_.iloc[valid_idx]
+       
+       args_cv = { 'X_train':X_train_cv, 'y_train': y_train_cv, 'X_test': X_valid_cv, 'y_test': y_valid_cv, 'params':params,
+                   "metric":metric,"num_rounds":num_rounds,"early_stop":None,"log":0,"random_state":random_state,"api":api ,
+                   "strategy":None, "url":None }  
+       
+       model_cv ,  y_prob_uno_train_cv, y_prob_uno_test_cv    = l.modelar(**args_cv)
+       
+       path_cv_test_filename = os.path.join(path_cv, f"y_test_cv_{i}.csv")      
+       df_ = pd.DataFrame()
+       df_["y_test_cv"] = y_valid_cv
+       df_["y_prob_uno_test_cv"] = y_prob_uno_test_cv  
+       df_.to_csv(path_cv_test_filename,index=False)  
+       
+   
+       path_cv_train_filename = os.path.join(path_cv, f"y_train_cv_{i}.csv") 
+       df_ = pd.DataFrame()
+       df_["y_train_cv"] = y_train_cv
+       df_["y_prob_uno_train_cv"] = y_prob_uno_train_cv       
+       df_.to_csv(path_cv_train_filename,index=False)     
+          
+       i += 1 
+       
+             
+
+def kpis_cross_validation():
+    
+    list_train_precision = []
+    list_train_recall = []
+    list_train_f1 = []
+    list_train_average_precision = []
+    list_train_roc_auc = []
+    
+    
+    list_valid_precision = []
+    list_valid_recall = []
+    list_valid_f1 = []
+    list_valid_average_precision = []
+    list_valid_roc_auc = []
+    
+    path_cv = "CV"
+    
+    for i in range(5):
+        
+    
+       df_cv_fold = pd.read_csv(f"{path_cv}/y_train_cv_{i}.csv", index_col=False)
+       y_train_cv = df_cv_fold.y_train_cv
+       y_prob_uno_train_cv = df_cv_fold.y_prob_uno_train_cv       
+       y_pred_train_cv = np.round(y_prob_uno_train_cv, 0)
+       
+       precision_train, recall_train, f1_train, support_train = precision_recall_fscore_support(y_train_cv, y_pred_train_cv ,average="binary",pos_label=1)
+       train_average_precision = average_precision_score(y_train_cv, y_prob_uno_train_cv)      
+       roc_auc_train = roc_auc_score(y_train_cv, y_prob_uno_train_cv) 
+        
+       list_train_precision.append(precision_train)
+       list_train_recall.append(recall_train)
+       list_train_f1.append(f1_train)
+       list_train_average_precision.append(train_average_precision)
+       list_train_roc_auc.append(roc_auc_train)
+       
+       
+       df_cv_fold = pd.read_csv(f"{path_cv}/y_test_cv_{i}.csv", index_col=False)
+       y_test_cv = df_cv_fold.y_test_cv
+       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv       
+       y_pred_valid_cv = np.round(y_prob_uno_test_cv, 0)
+       
+       precision_test, recall_test, f1_test, support_test = precision_recall_fscore_support(y_test_cv, y_pred_valid_cv ,average="binary",pos_label=1)
+       valid_average_precision = average_precision_score(y_test_cv, y_prob_uno_test_cv)      
+       roc_auc_valid = roc_auc_score(y_test_cv, y_prob_uno_test_cv) 
+        
+       list_valid_precision.append(precision_test)
+       list_valid_recall.append(recall_test)
+       list_valid_f1.append(f1_test)
+       list_valid_average_precision.append(valid_average_precision)
+       list_valid_roc_auc.append(roc_auc_valid)
+       
+   
+
+    mean_train_precision = statistics.mean(list_train_precision)           
+    std_train_precision = statistics.pstdev(list_train_precision)
+    mean_train_recall = statistics.mean(list_train_recall)           
+    std_train_recall = statistics.pstdev(list_train_recall)
+    mean_train_f1 = statistics.mean(list_train_f1)           
+    std_train_f1 = statistics.pstdev(list_train_f1)    
+    mean_train_average_precision = statistics.mean(list_train_average_precision) 
+    std_train_average_precision = statistics.pstdev(list_train_average_precision) 
+    mean_train_roc_auc = statistics.mean(list_train_roc_auc)           
+    std_train_roc_auc = statistics.pstdev(list_train_roc_auc)
+             
+    mean_valid_precision = statistics.mean(list_valid_precision)           
+    std_valid_precision = statistics.pstdev(list_valid_precision) 
+    mean_valid_recall = statistics.mean(list_valid_recall)           
+    std_valid_recall = statistics.pstdev(list_valid_recall)
+    mean_valid_f1 = statistics.mean(list_valid_f1)           
+    std_valid_f1 = statistics.pstdev(list_valid_f1)
+    mean_valid_average_precision = statistics.mean(list_valid_average_precision) 
+    std_valid_average_precision = statistics.pstdev(list_valid_average_precision)
+    mean_valid_roc_auc = statistics.mean(list_valid_roc_auc)           
+    std_valid_roc_auc = statistics.pstdev(list_valid_roc_auc)
+   
+    result_kpis = {   
+        
+        'mean_train_precision':mean_train_precision,
+        'std_train_precision':std_train_precision,
+        'mean_train_recall':mean_train_recall,
+        'std_train_recall':std_train_recall, 
+        'mean_train_f1':mean_train_f1,
+        'std_train_f1':std_train_f1 ,
+        'mean_train_average_precision':mean_train_average_precision,
+        'std_train_average_precision':std_train_average_precision,       
+        'mean_train_roc_auc':mean_train_roc_auc,
+        'std_train_roc_auc':std_train_roc_auc   ,               
+
+        'mean_valid_precision':mean_valid_precision,
+        'std_valid_precision':std_valid_precision,
+        'mean_valid_recall':mean_valid_recall,
+        'std_valid_recall':std_valid_recall, 
+        'mean_valid_f1':mean_valid_f1,
+        'std_valid_f1':std_valid_f1 ,
+        'mean_valid_average_precision':mean_valid_average_precision,
+        'std_valid_average_precision':std_valid_average_precision,       
+        'mean_valid_roc_auc':mean_valid_roc_auc,
+        'std_valid_roc_auc':std_valid_roc_auc   ,        
+    
+                
+    }  
+     
+    return result_kpis
+
+
+def roc_curva_cross_validation(list_path=[], n_splits=5):
+    
+    tprs = []
+    aucs = []
+    mean_fpr = np.linspace(0, 1, 100)
+
+    #list_path = ["CV","CV2"]
+    
+    for i in range(n_splits):
+    
+       df_cv_fold = pd.DataFrame()
+       for path_ in list_path:
+           print(" path_ :  ", path_)
+           df_cv_fold_ = pd.read_csv(f"{path_}/y_test_cv_{i}.csv", index_col=False)
+           df_cv_fold = df_cv_fold.append(df_cv_fold_)
+           
+       y_test_cv = df_cv_fold.y_test_cv
+       y_prob_uno_test_cv = df_cv_fold.y_prob_uno_test_cv
+       
+       fpr, tpr, thresholds = roc_curve(y_test_cv, y_prob_uno_test_cv)
+       tprs.append(interp(mean_fpr, fpr, tpr))
+       tprs[-1][0] = 0.0
+       roc_auc = auc(fpr, tpr)
+       aucs.append(roc_auc)
+       #plt.plot(fpr, tpr, lw=1, alpha=0.3, label='ROC fold %d (AUC = %0.2f)' % (i, roc_auc))
+       i += 1  
+    
+    
+    plt.plot([0, 1], [0, 1], linestyle='--', lw=2, color='r', label='Luck', alpha=.8)  
+   
+    mean_tpr = np.mean(tprs, axis=0)
+    mean_tpr[-1] = 1.0
+    mean_auc = auc(mean_fpr, mean_tpr)
+    std_auc = np.std(aucs)
+    plt.plot(mean_fpr, mean_tpr, color='b',label=r'Mean ROC (AUC = %0.2f $\pm$ %0.2f)' % (mean_auc, std_auc),lw=1, alpha=.8)
+   
+    std_tpr = np.std(tprs, axis=0)
+    tprs_upper = np.minimum(mean_tpr + std_tpr, 1)
+    tprs_lower = np.maximum(mean_tpr - std_tpr, 0)
+    plt.fill_between(mean_fpr, tprs_lower, tprs_upper, color='grey', alpha=.5, label=r'$\pm$ 1 std. dev.')
+
+    plt.xlim([-0.05, 1.05])
+    plt.ylim([-0.05, 1.05])
+    plt.xlabel('False Positive Rate')
+    plt.ylabel('True Positive Rate')
+    plt.title("Curva ROC")
+    plt.legend(loc="lower right")
+    plt.show()
+
+
 def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=None,y_test=None,score_rs=None,params=None,param_dist=None, n_iter=None,n_jobs=None,url=None,print_consola=True):
     if url is not None:
         hg.validar_directorio(url)     
@@ -174,6 +406,7 @@ def modelar_clasificacion_binaria_rscv(strategy, X_train,y_train=None,X_test=Non
     
 
 def generar_reporte(model,y_prob_uno, X_test, y_test,umbral_opt,url,print_consola): 
+    
      
     kpis = hp.print_kpis_rendimiento_modelo(y_test,y_prob_uno,umbral_opt,url,print_consola)   
     '''
